@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2016 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
   Copyright 2008 Krzysztof Foltman
 
   Permission to use, copy, modify, and/or distribute this software for any
@@ -18,16 +18,7 @@
 #define _POSIX_C_SOURCE 200809L /* for setenv */
 #define _XOPEN_SOURCE   600     /* for mkstemp */
 
-#include <assert.h>
-#include <ctype.h>
-#include <errno.h>
-#include <float.h>
-#include <limits.h>
-#include <math.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/stat.h>
+#include "../src/lilv_internal.h"
 
 #ifdef _WIN32
 #    include <direct.h>
@@ -37,16 +28,27 @@
 #    define unsetenv(n) SetEnvironmentVariable((n), NULL)
 #    define mkstemp(pat) _mktemp(pat)
 #else
-#    include <dirent.h>
 #    include <unistd.h>
 #endif
 
 #include "lilv/lilv.h"
-#include "../src/lilv_internal.h"
+#include "lv2/core/lv2.h"
+#include "lv2/presets/presets.h"
+#include "lv2/state/state.h"
+#include "lv2/urid/urid.h"
+#include "serd/serd.h"
 
-#include "lv2/lv2plug.in/ns/ext/presets/presets.h"
-#include "lv2/lv2plug.in/ns/ext/state/state.h"
-#include "lv2/lv2plug.in/ns/ext/urid/urid.h"
+#include <assert.h>
+#include <errno.h>
+#include <float.h>
+#include <math.h>
+#include <stdarg.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
 
 #define TEST_PATH_MAX 1024
 
@@ -58,10 +60,10 @@
 #    define SHLIB_EXT ".so"
 #endif
 
-static char bundle_dir_name[TEST_PATH_MAX];
-static char bundle_dir_uri[TEST_PATH_MAX];
-static char manifest_name[TEST_PATH_MAX];
-static char content_name[TEST_PATH_MAX];
+static char test_bundle_path[TEST_PATH_MAX + sizeof("/.lv2/lilv-test.lv2")];
+static char test_bundle_uri[sizeof(test_bundle_path) + sizeof("file:///")];
+static char test_manifest_path[sizeof(test_bundle_path) + sizeof("/manifest.ttl")];
+static char test_content_path[sizeof(test_bundle_path) + sizeof("plugin.ttl")];
 
 static LilvWorld* world;
 
@@ -71,32 +73,44 @@ int error_count = 0;
 static void
 delete_bundle(void)
 {
-	unlink(content_name);
-	unlink(manifest_name);
-	remove(bundle_dir_name);
+	unlink(test_content_path);
+	unlink(test_manifest_path);
+	remove(test_bundle_path);
 }
 
 static void
 init_tests(void)
 {
-	snprintf(bundle_dir_name, TEST_PATH_MAX, "%s/.lv2/lilv-test.lv2",
-	         getenv("HOME"));
-	lilv_mkdir_p(bundle_dir_name);
+	char* test_path = lilv_realpath(LILV_TEST_DIR);
 
-	snprintf(bundle_dir_uri, TEST_PATH_MAX, "file://%s/", bundle_dir_name);
-	snprintf(manifest_name, TEST_PATH_MAX, "%s/manifest.ttl", bundle_dir_name);
-	snprintf(content_name, TEST_PATH_MAX, "%s/plugin.ttl", bundle_dir_name);
+	snprintf(test_bundle_path, sizeof(test_bundle_path),
+	         "%s/test_lv2_path/lilv-test.lv2", test_path);
+	lilv_mkdir_p(test_bundle_path);
+
+	SerdNode s = serd_node_new_file_uri(
+		(const uint8_t*)test_bundle_path, NULL, NULL, true);
+
+	snprintf(test_bundle_uri, sizeof(test_bundle_uri), "%s/",
+	         (const char*)s.buf);
+	snprintf(test_manifest_path, sizeof(test_manifest_path), "%s/manifest.ttl",
+	         test_bundle_path);
+	snprintf(test_content_path, sizeof(test_content_path), "%s/plugin.ttl",
+	         test_bundle_path);
+
+	serd_node_free(&s);
+	lilv_free(test_path);
 
 	delete_bundle();
 }
 
 static void
-fatal_error(const char* err, const char* arg)
+fatal_error(const char* fmt, ...)
 {
-	/* TODO: possibly change to vfprintf later */
-	fprintf(stderr, err, arg);
-	/* IMHO, the bundle should be left in place after an error, for possible investigation */
-	/* delete_bundle(); */
+	va_list args;
+	va_start(args, fmt);
+	fprintf(stderr, "error: ");
+	vfprintf(stderr, fmt, args);
+	va_end(args);
 	exit(1);
 }
 
@@ -106,7 +120,8 @@ write_file(const char* name, const char* content)
 	FILE* f = fopen(name, "w");
 	size_t len = strlen(content);
 	if (fwrite(content, 1, len, f) != len) {
-		fatal_error("Cannot write file %s\n", name);
+		fatal_error("Failed to write to file '%s' (%s)\n",
+		            name, strerror(errno));
 	}
 	fclose(f);
 }
@@ -115,6 +130,16 @@ static int
 init_world(void)
 {
 	world = lilv_world_new();
+
+	// Set custom LV2_PATH in build directory to only use test data
+	char*     test_path = lilv_realpath(LILV_TEST_DIR);
+	char*     lv2_path  = lilv_strjoin(test_path, "/test_lv2_path", NULL);
+	LilvNode* path      = lilv_new_string(world, lv2_path);
+	lilv_world_set_option(world, LILV_OPTION_LV2_PATH, path);
+	free(lv2_path);
+	free(test_path);
+	lilv_node_free(path);
+
 	return world != NULL;
 }
 
@@ -131,11 +156,12 @@ load_all_bundles(void)
 static void
 create_bundle(const char* manifest, const char* content)
 {
-	if (mkdir(bundle_dir_name, 0700) && errno != EEXIST) {
-		fatal_error("Cannot create directory %s\n", bundle_dir_name);
+	if (mkdir(test_bundle_path, 0700) && errno != EEXIST) {
+		fatal_error("Failed to create directory '%s' (%s)\n",
+		            test_bundle_path, strerror(errno));
 	}
-	write_file(manifest_name, manifest);
-	write_file(content_name, content);
+	write_file(test_manifest_path, manifest);
+	write_file(test_content_path, content);
 }
 
 static int
@@ -160,6 +186,21 @@ cleanup(void)
 	delete_bundle();
 }
 
+static void
+set_env(const char* name, const char* value)
+{
+#ifdef _WIN32
+	// setenv on Windows does not modify the current process' environment
+	const size_t len = strlen(name) + 1 + strlen(value) + 1;
+	char*        str = (char*)calloc(1, len);
+	snprintf(str, len, "%s=%s", name, value);
+	putenv(str);
+	free(str);
+#else
+	setenv(name, value, 1);
+#endif
+}
+
 /*****************************************************************************/
 
 #define TEST_CASE(name) { #name, test_##name }
@@ -461,42 +502,6 @@ test_discovery(void)
 
 /*****************************************************************************/
 
-static int
-test_lv2_path(void)
-{
-#ifndef _WIN32
-	char* orig_lv2_path = lilv_strdup(getenv("LV2_PATH"));
-
-	setenv("LV2_PATH", "~/.lv2:/usr/local/lib/lv2:/usr/lib/lv2", 1);
-
-	world = lilv_world_new();
-	lilv_world_load_all(world);
-
-	const LilvPlugins* plugins = lilv_world_get_all_plugins(world);
-	const size_t n_plugins     = lilv_plugins_size(plugins);
-
-	lilv_world_free(world);
-
-	setenv("LV2_PATH", "$HOME/.lv2:/usr/local/lib/lv2:/usr/lib/lv2", 1);
-	world = lilv_world_new();
-	lilv_world_load_all(world);
-	plugins = lilv_world_get_all_plugins(world);
-	TEST_ASSERT(lilv_plugins_size(plugins) == n_plugins);
-	lilv_world_free(world);
-	world = NULL;
-
-	if (orig_lv2_path) {
-		setenv("LV2_PATH", orig_lv2_path, 1);
-	} else {
-		unsetenv("LV2_PATH");
-	}
-	free(orig_lv2_path);
-#endif
-	return 1;
-}
-
-/*****************************************************************************/
-
 static int
 test_verify(void)
 {
@@ -582,6 +587,8 @@ test_classes(void)
 
 	lilv_plugin_classes_free(children);
 
+	lilv_plugin_class_free(NULL);
+
 	cleanup_uris();
 	return 1;
 }
@@ -646,7 +653,7 @@ test_plugin(void)
 	TEST_ASSERT(!lilv_plugin_get_related(plug, NULL));
 
 	const LilvNode* plug_bundle_uri = lilv_plugin_get_bundle_uri(plug);
-	TEST_ASSERT(!strcmp(lilv_node_as_string(plug_bundle_uri), bundle_dir_uri));
+	TEST_ASSERT(!strcmp(lilv_node_as_string(plug_bundle_uri), test_bundle_uri));
 
 	const LilvNodes* data_uris = lilv_plugin_get_data_uris(plug);
 	TEST_ASSERT(lilv_nodes_size(data_uris) == 2);
@@ -1092,7 +1099,7 @@ test_port(void)
 			"  a lv2:ControlPort ; a lv2:InputPort ; "
 			"  lv2:index 0 ; lv2:symbol \"foo\" ; "
 			"  lv2:name \"store\" ; "
-			"  lv2:name \"dépanneur\"@fr-ca ; lv2:name \"épicerie\"@fr-fr ; "
+			"  lv2:name \"Laden\"@de-de ; lv2:name \"Geschaeft\"@de-at ; "
 			"  lv2:name \"tienda\"@es ; "
 			"  rdfs:comment \"comment\"@en , \"commentaires\"@fr ; "
      		"  lv2:portProperty lv2:integer ; "
@@ -1167,43 +1174,43 @@ test_port(void)
 	lilv_node_free(name);
 
 	// Exact language match
-	setenv("LANG", "fr_FR", 1);
+	set_env("LANG", "de_DE");
 	name = lilv_port_get_name(plug, p);
-	TEST_ASSERT(!strcmp(lilv_node_as_string(name), "épicerie"));
+	TEST_ASSERT(!strcmp(lilv_node_as_string(name), "Laden"));
 	lilv_node_free(name);
 
 	// Exact language match (with charset suffix)
-	setenv("LANG", "fr_CA.utf8", 1);
+	set_env("LANG", "de_AT.utf8");
 	name = lilv_port_get_name(plug, p);
-	TEST_ASSERT(!strcmp(lilv_node_as_string(name), "dépanneur"));
+	TEST_ASSERT(!strcmp(lilv_node_as_string(name), "Geschaeft"));
 	lilv_node_free(name);
 
 	// Partial language match (choose value translated for different country)
-	setenv("LANG", "fr_BE", 1);
+	set_env("LANG", "de_CH");
 	name = lilv_port_get_name(plug, p);
-	TEST_ASSERT((!strcmp(lilv_node_as_string(name), "dépanneur"))
-	            ||(!strcmp(lilv_node_as_string(name), "épicerie")));
+	TEST_ASSERT((!strcmp(lilv_node_as_string(name), "Laden"))
+	            ||(!strcmp(lilv_node_as_string(name), "Geschaeft")));
 	lilv_node_free(name);
 
 	// Partial language match (choose country-less language tagged value)
-	setenv("LANG", "es_MX", 1);
+	set_env("LANG", "es_MX");
 	name = lilv_port_get_name(plug, p);
 	TEST_ASSERT(!strcmp(lilv_node_as_string(name), "tienda"));
 	lilv_node_free(name);
 
 	// No language match (choose untranslated value)
-	setenv("LANG", "cn", 1);
+	set_env("LANG", "cn");
 	name = lilv_port_get_name(plug, p);
 	TEST_ASSERT(!strcmp(lilv_node_as_string(name), "store"));
 	lilv_node_free(name);
 
 	// Invalid language
-	setenv("LANG", "1!", 1);
+	set_env("LANG", "1!");
 	name = lilv_port_get_name(plug, p);
 	TEST_ASSERT(!strcmp(lilv_node_as_string(name), "store"));
 	lilv_node_free(name);
 
-	setenv("LANG", "en_CA.utf-8", 1);
+	set_env("LANG", "en_CA.utf-8");
 
 	// Language tagged value with no untranslated values
 	LilvNode*  rdfs_comment = lilv_new_uri(world, LILV_NS_RDFS "comment");
@@ -1215,14 +1222,14 @@ test_port(void)
 	lilv_node_free(comment);
 	lilv_nodes_free(comments);
 
-	setenv("LANG", "fr", 1);
+	set_env("LANG", "fr");
 
 	comments = lilv_port_get_value(plug, p, rdfs_comment);
 	TEST_ASSERT(!strcmp(lilv_node_as_string(lilv_nodes_get_first(comments)),
 	                    "commentaires"));
 	lilv_nodes_free(comments);
 
-	setenv("LANG", "cn", 1);
+	set_env("LANG", "cn");
 
 	comments = lilv_port_get_value(plug, p, rdfs_comment);
 	TEST_ASSERT(!comments);
@@ -1230,7 +1237,7 @@ test_port(void)
 
 	lilv_node_free(rdfs_comment);
 
-	setenv("LANG", "C", 1);  // Reset locale
+	set_env("LANG", "C");  // Reset locale
 
 	LilvScalePoints* points = lilv_port_get_scale_points(plug, p);
 	TEST_ASSERT(lilv_scale_points_size(points) == 2);
@@ -1475,6 +1482,8 @@ test_ui(void)
 
 /*****************************************************************************/
 
+#ifndef _WIN32
+
 uint32_t atom_Float = 0;
 float    in         = 1.0;
 float    out        = 42.0;
@@ -1563,6 +1572,12 @@ lilv_make_path(LV2_State_Make_Path_Handle handle,
 	return lilv_path_join(temp_dir, path);
 }
 
+static void
+lilv_free_path(LV2_State_Free_Path_Handle handle, char* path)
+{
+	lilv_free(path);
+}
+
 static int
 test_state(void)
 {
@@ -1604,21 +1619,21 @@ test_state(void)
 
 	temp_dir = lilv_realpath("temp");
 
-	const char* file_dir = NULL;
-	char*       copy_dir = NULL;
-	char*       link_dir = NULL;
-	char*       save_dir = NULL;
+	const char* scratch_dir = NULL;
+	char*       copy_dir    = NULL;
+	char*       link_dir    = NULL;
+	char*       save_dir    = NULL;
 
 	// Get instance state state
 	LilvState* state = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, save_dir,
+		scratch_dir, copy_dir, link_dir, save_dir,
 		get_port_value, world, 0, NULL);
 
 	// Get another instance state
 	LilvState* state2 = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, save_dir,
+		scratch_dir, copy_dir, link_dir, save_dir,
 		get_port_value, world, 0, NULL);
 
 	// Ensure they are equal
@@ -1648,7 +1663,7 @@ test_state(void)
 
 	// Ensure they are equal
 	TEST_ASSERT(lilv_state_equals(state, from_str));
-	free(state1_str);
+	lilv_free(state1_str);
 
 	const LilvNode* state_plugin_uri = lilv_state_get_plugin_uri(state);
 	TEST_ASSERT(lilv_node_equals(state_plugin_uri, plugin_uri));
@@ -1666,7 +1681,7 @@ test_state(void)
 	lilv_instance_run(instance, 1);
 	LilvState* state3 = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, save_dir,
+		scratch_dir, copy_dir, link_dir, save_dir,
 		get_port_value, world, 0, NULL);
 	TEST_ASSERT(!lilv_state_equals(state2, state3));  // num_runs changed
 
@@ -1676,7 +1691,7 @@ test_state(void)
 	// Take a new snapshot and ensure it matches the set state
 	LilvState* state4 = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, save_dir,
+		scratch_dir, copy_dir, link_dir, save_dir,
 		get_port_value, world, 0, NULL);
 	TEST_ASSERT(lilv_state_equals(state2, state4));
 
@@ -1719,39 +1734,42 @@ test_state(void)
 	                      "state/state6.lv2", "state6.ttl");
 	TEST_ASSERT(!ret);
 
-	// Load default bundle into world and load state from it
-	uint8_t*  state6_path       = (uint8_t*)lilv_path_absolute("state/state6.lv2/");
-	SerdNode  state6_uri        = serd_node_new_file_uri(state6_path, 0, 0, true);
-	LilvNode* test_state_bundle = lilv_new_uri(world, (const char*)state6_uri.buf);
-	LilvNode* test_state_node   = lilv_new_uri(world, state_uri);
-	lilv_world_load_bundle(world, test_state_bundle);
-	lilv_world_load_resource(world, test_state_node);
-	serd_node_free(&state6_uri);
-	free(state6_path);
+	// Load state bundle into world and verify it matches
+	{
+		uint8_t*  state6_path   = (uint8_t*)lilv_path_absolute("state/state6.lv2/");
+		SerdNode  state6_uri    = serd_node_new_file_uri(state6_path, 0, 0, true);
+		LilvNode* state6_bundle = lilv_new_uri(world, (const char*)state6_uri.buf);
+		LilvNode* state6_node   = lilv_new_uri(world, state_uri);
+		lilv_world_load_bundle(world, state6_bundle);
+		lilv_world_load_resource(world, state6_node);
+		serd_node_free(&state6_uri);
+		lilv_free(state6_path);
 
-	LilvState* state6 = lilv_state_new_from_world(world, &map, test_state_node);
-	TEST_ASSERT(lilv_state_equals(state, state6));  // Round trip accuracy
+		LilvState* state6 = lilv_state_new_from_world(world, &map, state6_node);
+		TEST_ASSERT(lilv_state_equals(state, state6));  // Round trip accuracy
 
-	// Check that loaded state has correct URI
-	TEST_ASSERT(lilv_state_get_uri(state6));
-	TEST_ASSERT(!strcmp(lilv_node_as_string(lilv_state_get_uri(state6)),
-	                    state_uri));
+		// Check that loaded state has correct URI
+		TEST_ASSERT(lilv_state_get_uri(state6));
+		TEST_ASSERT(!strcmp(lilv_node_as_string(lilv_state_get_uri(state6)),
+		                    state_uri));
 
-	lilv_world_unload_resource(world, test_state_node);
-	lilv_world_unload_bundle(world, test_state_bundle);
+		// Unload state from world
+		lilv_world_unload_resource(world, state6_node);
+		lilv_world_unload_bundle(world, state6_bundle);
 
-	LilvState* state6_2 = lilv_state_new_from_world(world, &map, test_state_node);
-	TEST_ASSERT(!state6_2);  // No longer present
-	lilv_state_free(state6_2);
+		// Ensure that it is no longer present
+		TEST_ASSERT(!lilv_state_new_from_world(world, &map, state6_node));
+		lilv_node_free(state6_bundle);
+		lilv_node_free(state6_node);
 
-	lilv_node_free(test_state_bundle);
-	lilv_node_free(test_state_node);
-
-	unsetenv("LV2_STATE_BUNDLE");
+		// Delete state
+		lilv_state_delete(world, state6);
+		lilv_state_free(state6);
+	}
 
 	// Make directories and test files support
 	mkdir("temp", 0700);
-	file_dir = temp_dir;
+	scratch_dir = temp_dir;
 	mkdir("files", 0700);
 	copy_dir = lilv_realpath("files");
 	mkdir("links", 0700);
@@ -1759,7 +1777,11 @@ test_state(void)
 
 	LV2_State_Make_Path make_path         = { NULL, lilv_make_path };
 	LV2_Feature         make_path_feature = { LV2_STATE__makePath, &make_path };
-	const LV2_Feature*  ffeatures[]       = { &make_path_feature, &map_feature, NULL };
+	LV2_State_Free_Path free_path         = { NULL, lilv_free_path };
+	LV2_Feature         free_path_feature = { LV2_STATE__freePath, &free_path };
+	const LV2_Feature*  ffeatures[]       = {
+		&make_path_feature, &map_feature, &free_path_feature, NULL
+	};
 
 	lilv_instance_deactivate(instance);
 	lilv_instance_free(instance);
@@ -1781,24 +1803,30 @@ test_state(void)
 	// Get instance state state
 	LilvState* fstate = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, "state/fstate.lv2",
+		scratch_dir, copy_dir, link_dir, "state/fstate.lv2",
 		get_port_value, world, 0, ffeatures);
 
-	// Get another instance state
-	LilvState* fstate2 = lilv_state_new_from_instance(
-		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, "state/fstate2.lv2",
-		get_port_value, world, 0, ffeatures);
+	{
+		// Get another instance state
+		LilvState* fstate2 = lilv_state_new_from_instance(
+			plugin, instance, &map,
+			scratch_dir, copy_dir, link_dir, "state/fstate2.lv2",
+			get_port_value, world, 0, ffeatures);
 
-	// Should be identical
-	TEST_ASSERT(lilv_state_equals(fstate, fstate2));
+		// Check that it is identical
+		TEST_ASSERT(lilv_state_equals(fstate, fstate2));
+
+		lilv_state_delete(world, fstate2);
+		lilv_state_free(fstate2);
+	}
 
 	// Run, writing more to rec file
 	lilv_instance_run(instance, 2);
 
 	// Get yet another instance state
 	LilvState* fstate3 = lilv_state_new_from_instance(
-		plugin, instance, &map, file_dir, copy_dir, link_dir, "state/fstate3.lv2",
+		plugin, instance, &map,
+		scratch_dir, copy_dir, link_dir, "state/fstate3.lv2",
 		get_port_value, world, 0, ffeatures);
 
 	// Should be different
@@ -1810,8 +1838,8 @@ test_state(void)
 	TEST_ASSERT(!ret);
 
 	// Load state from directory
-	LilvState* fstate4 = lilv_state_new_from_file(world, &map, NULL,
-	                                              "state/fstate.lv2/fstate.ttl");
+	LilvState* fstate4 = lilv_state_new_from_file(
+		world, &map, NULL, "state/fstate.lv2/fstate.ttl");
 	TEST_ASSERT(lilv_state_equals(fstate, fstate4));  // Round trip accuracy
 
 	// Restore instance state to loaded state
@@ -1820,7 +1848,7 @@ test_state(void)
 	// Take a new snapshot and ensure it matches
 	LilvState* fstate5 = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, "state/fstate5.lv2",
+		scratch_dir, copy_dir, link_dir, "state/fstate5.lv2",
 		get_port_value, world, 0, ffeatures);
 	TEST_ASSERT(lilv_state_equals(fstate3, fstate5));
 
@@ -1830,8 +1858,8 @@ test_state(void)
 	TEST_ASSERT(!ret);
 
 	// Reload it and ensure it's identical to the other loaded version
-	LilvState* fstate6 = lilv_state_new_from_file(world, &map, NULL,
-	                                              "state/fstate6.lv2/fstate6.ttl");
+	LilvState* fstate6 = lilv_state_new_from_file(
+		world, &map, NULL, "state/fstate6.lv2/fstate6.ttl");
 	TEST_ASSERT(lilv_state_equals(fstate4, fstate6));
 
 	// Run, changing rec file (without changing size)
@@ -1840,7 +1868,7 @@ test_state(void)
 	// Take a new snapshot
 	LilvState* fstate7 = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, "state/fstate7.lv2",
+		scratch_dir, copy_dir, link_dir, "state/fstate7.lv2",
 		get_port_value, world, 0, ffeatures);
 	TEST_ASSERT(!lilv_state_equals(fstate6, fstate7));
 
@@ -1850,13 +1878,33 @@ test_state(void)
 	TEST_ASSERT(!ret);
 
 	// Reload it and ensure it's changed
-	LilvState* fstate72 = lilv_state_new_from_file(world, &map, NULL,
-	                                              "state/fstate7.lv2/fstate7.ttl");
+	LilvState* fstate72 = lilv_state_new_from_file(
+		world, &map, NULL, "state/fstate7.lv2/fstate7.ttl");
 	TEST_ASSERT(lilv_state_equals(fstate72, fstate7));
 	TEST_ASSERT(!lilv_state_equals(fstate6, fstate72));
 
-	// Delete saved state
+	// Delete saved state we still have a state in memory that points to
 	lilv_state_delete(world, fstate7);
+	lilv_state_delete(world, fstate6);
+	lilv_state_delete(world, fstate5);
+	lilv_state_delete(world, fstate3);
+	lilv_state_delete(world, fstate);
+	lilv_state_delete(world, state2);
+	lilv_state_delete(world, state);
+
+	// Delete remaining states on disk we've lost a reference to
+	const char* const old_state_paths[] = {"state/state.lv2/state.ttl",
+	                                       "state/state.lv2/state2.ttl",
+	                                       "state/fstate.lv2/fstate.ttl",
+	                                       NULL};
+
+	for (const char*const* p = old_state_paths; *p; ++p) {
+		const char* path = *p;
+		LilvState*  old_state =
+			lilv_state_new_from_file(world, &map, NULL, path);
+		lilv_state_delete(world, old_state);
+		lilv_state_free(old_state);
+	}
 
 	lilv_instance_deactivate(instance);
 	lilv_instance_free(instance);
@@ -1869,9 +1917,7 @@ test_state(void)
 	lilv_state_free(state3);
 	lilv_state_free(state4);
 	lilv_state_free(state5);
-	lilv_state_free(state6);
 	lilv_state_free(fstate);
-	lilv_state_free(fstate2);
 	lilv_state_free(fstate3);
 	lilv_state_free(fstate4);
 	lilv_state_free(fstate5);
@@ -1879,6 +1925,8 @@ test_state(void)
 	lilv_state_free(fstate7);
 	lilv_state_free(fstate72);
 
+	rmdir("state");
+
 	// Free URI map
 	for (size_t i = 0; i < n_uris; ++i) {
 		free(uris[i]);
@@ -1895,6 +1943,7 @@ test_state(void)
 	cleanup_uris();
 	return 1;
 }
+#endif
 
 /*****************************************************************************/
 
@@ -2050,7 +2099,7 @@ test_reload_bundle(void)
 	lilv_world_load_specifications(world);
 
 	// Load bundle
-	LilvNode* bundle_uri = lilv_new_uri(world, bundle_dir_uri);
+	LilvNode* bundle_uri = lilv_new_uri(world, test_bundle_uri);
 	lilv_world_load_bundle(world, bundle_uri);
 
 	// Check that plugin is present
@@ -2238,7 +2287,6 @@ static struct TestCase tests[] = {
 	TEST_CASE(verify),
 	TEST_CASE(no_verify),
 	TEST_CASE(discovery),
-	TEST_CASE(lv2_path),
 	TEST_CASE(classes),
 	TEST_CASE(plugin),
 	TEST_CASE(project),
@@ -2253,7 +2301,10 @@ static struct TestCase tests[] = {
 	TEST_CASE(bad_port_index),
 	TEST_CASE(string),
 	TEST_CASE(world),
+	// FIXME: State is not currently working on Windows
+#ifndef _WIN32
 	TEST_CASE(state),
+#endif
 	TEST_CASE(reload_bundle),
 	TEST_CASE(replace_version),
 	TEST_CASE(get_symbol),
@@ -2284,7 +2335,7 @@ main(int argc, char* argv[])
 		printf("Syntax: %s\n", argv[0]);
 		return 0;
 	}
-	setenv("LANG", "C", 1);
+	set_env("LANG", "C");
 	init_tests();
 	run_tests();
 	cleanup();
